// Licensed to the Apache Software Foundation (ASF) under one
// or more contributor license agreements.  See the NOTICE file
// distributed with this work for additional information
// regarding copyright ownership.  The ASF licenses this file
// to you under the Apache License, Version 2.0 (the
// "License"); you may not use this file except in compliance
// with the License.  You may obtain a copy of the License at
//
//   http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

#ifndef GANDIVA_FUNCTION_REGISTRY_H
#define GANDIVA_FUNCTION_REGISTRY_H

#include <unordered_map>

#include "gandiva/gandiva_aliases.h"
#include "gandiva/native_function.h"

namespace gandiva {

///\brief Registry of pre-compiled IR functions.
class FunctionRegistry {
 public:
  using iterator = const NativeFunction*;

  /// Lookup a pre-compiled function by its signature.
  const NativeFunction* LookupSignature(const FunctionSignature& signature) const;

  iterator begin() const;
  iterator end() const;

 private:
  struct KeyHash {
    std::size_t operator()(const FunctionSignature* k) const { return k->Hash(); }
  };

  struct KeyEquals {
    bool operator()(const FunctionSignature* s1, const FunctionSignature* s2) const {
      return *s1 == *s2;
    }
  };

  static DataTypePtr time32() { return arrow::time32(arrow::TimeUnit::MILLI); }

  static DataTypePtr time64() { return arrow::time64(arrow::TimeUnit::MICRO); }

  static DataTypePtr timestamp() { return arrow::timestamp(arrow::TimeUnit::MILLI); }

  typedef std::unordered_map<const FunctionSignature*, const NativeFunction*, KeyHash,
                             KeyEquals>
      SignatureMap;
  static SignatureMap InitPCMap();

  static NativeFunction pc_registry_[];
  static SignatureMap pc_registry_map_;
};

}  // namespace gandiva

#endif  // GANDIVA_FUNCTION_REGISTRY_H
